"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Placeholders = void 0;
class Placeholders {
    /**
     * @param {string} str
     * @param {any[]} placeholders
     * @param emptyPlaceholder
     * @returns {string}
     */
    static replace(str, placeholders, emptyPlaceholder = '') {
        let index = 0;
        return str.replace(/{([0-9]*)(:[0-9]+)?}/gi, (_, p, q) => {
            const trim_after = Number(q !== undefined && q !== '' ? q.split(':')[1] : -1);
            if (p === '')
                return Placeholders.preparePlaceholderValue(placeholders[index] !== undefined ? placeholders[index++] : emptyPlaceholder, trim_after);
            return Placeholders.preparePlaceholderValue(placeholders[Number(p)] !== undefined ? placeholders[Number(p)] : emptyPlaceholder, trim_after);
        });
    }
    /**
     * @param {string} str
     * @param placeholders
     * @param emptyPlaceholder
     * @returns {string}
     */
    static replaceNamed(str, placeholders, emptyPlaceholder = '') {
        return str.replace(/{([A-Za-z0-9_]+)(:[0-9]+)?}/gi, (_, p, q) => {
            const trim_after = Number(q !== undefined && q !== '' ? q.split(':')[1] : 0);
            if (placeholders[p] !== undefined && placeholders.hasOwnProperty(p))
                return Placeholders.preparePlaceholderValue(placeholders[p], trim_after);
            return Placeholders.preparePlaceholderValue(emptyPlaceholder, trim_after);
        });
    }
    /**
     * @param placeholder_value
     * @param {number} [trim_after=0]
     * @returns {string}
     */
    static preparePlaceholderValue(placeholder_value, trim_after = 0) {
        const placeholder_t = typeof placeholder_value;
        let str = '';
        switch (placeholder_t) {
            case 'undefined':
                str = 'undefined';
                break;
            case 'string':
                str = placeholder_value;
                break;
            case 'number':
            case 'boolean':
            case 'symbol':
                str = placeholder_value.toString();
                break;
            case 'bigint':
                str = `BigInt(${placeholder_value.toString()})`;
                break;
            case 'function':
                str = 'Function' + (placeholder_value.name !== '' ? '(' + placeholder_value.name + ')' : '');
                break;
            case 'object':
                {
                    if (placeholder_value instanceof Error) {
                        const props = [
                            `constructor=${placeholder_value.constructor.name}`,
                            `message="${placeholder_value.message}"`
                        ];
                        for (const prop of Object.keys(placeholder_value)) {
                            // @ts-ignore
                            props.push(`${prop}=${placeholder_value[prop]}`);
                        }
                        str = `Error(${props.join(', ')})`;
                    }
                    else if (placeholder_value instanceof RegExp) {
                        str = `RegExp(${placeholder_value.toString()})`;
                    }
                    else {
                        str = JSON.stringify(placeholder_value);
                    }
                }
                break;
        }
        return trim_after > 0 && str.length > trim_after + 3 ? str.substr(0, trim_after) + '...' : str;
    }
}
exports.Placeholders = Placeholders;
